import yaml
import re
from pathlib import Path
import shutil

ALLOWED_ASSETS = [".png", ".jpg", ".jpeg", ".svg"]
DEST_FOLDER = Path("content")
DEST_ASSETS = Path("static/img")

def compute_default_title(s):
    return s.replace("_", " ")

def read_file(f):
    with open(f, "r") as f:
        return f.read() 
    
def get_front_matter(title, weight):
    return f"""---
title: '{title}'
date: date:2018-11-28T15:14:39+10:00
weight: {weight}
---
"""

def maybe_remove_title(f):
    if f.remove_title:
        f.content = f.content.split("\n",1)[1]

def get_toc(f):
    return "{{< toc >}}\n" if f.toc else ""

def write_file(f):
    maybe_remove_title(f)
    f.content = get_front_matter(f.title, f.weight) + get_toc(f) + f.content
    with open(f.dest, "w") as _file:
        _file.write(f.content)
        print(f"writing {f.dest} with title {f.title}")

content = yaml.load(read_file("content.yaml"), Loader=yaml.FullLoader)
assets = yaml.load(read_file("assets.yaml"), Loader=yaml.FullLoader)

# Copy assets
assets_map = dict()
for k, v in assets.items():
    dest_path = DEST_ASSETS / v["dest"]
    dest_path.mkdir(parents=True, exist_ok=True)

    src_path = Path(v["src"])
    asset_files = src_path.glob("*")
    for f in asset_files:
        if f.suffix in ALLOWED_ASSETS:
            result = shutil.copy(f, dest_path)
            src_path = Path(v["src_link"]) / f.name
            tgt_path = Path("/") / Path(*Path(result).parts[1:])
            assets_map[str(src_path)] = str(tgt_path)

def parse_nbdev_output(content):
    summary = re.search(r"summary:.*", content)

    content = re.sub(r"---[^---]*---", "", content)
    if summary:
        content = "**" + summary.group(0).replace('summary: "', "")[:-1] + "**" + content
    content = re.sub(r'<a class="anchor-link"[^<]*</a>', "", content)
    content = re.sub(f'{{% raw %}}', "", content)
    content = re.sub(f'{{% endraw %}}', "", content)
    content = re.sub(f'<pre><code>', '<pre><code class="nbdev-code chroma">', content)
    content = re.sub(f'<div class="highlight"><pre>', f'<div class="highlight"><pre class="nbdev-code chroma">', content)
    content = re.sub(r'\[source\]', f'', content)
    content = re.sub(r'\n\n', f"\n", content)
    content = re.sub(r'<pre><span></span>', f'<pre class="nbdev-code chroma"><span></span>', content)
    content = re.sub(r'<a href="https://gitlab.memri.io/memri/pymemri[^<]*</a>', "", content)
    content = re.sub(r'hl-ipython3', f"hl-ipython3 chroma", content)

    return content

def replace_mermaid(f):
    # remove double \n
    for i in range(5):
        f.content = re.sub(r"```mermaid([^`]*)\n\n([^`]*)```",r"```mermaid\1\n\2```", f.content)
    f.content = re.sub(r"```mermaid([^`]*)```",r'<div class="mermaid">\1</div>', f.content)  
  
link_in2out = dict()


class File():
    def __init__(self, content, src, dest, title, weight, toc=False, remove_title=False):
        self.content = content
        self.src = src
        self.dest = dest
        self.title = title
        self.weight = weight
        self.toc = toc
        self.lib = str(self.src).split("/")[0]
        self.remove_title=remove_title
        
class Lib():
    def __init__(self, name):
        self.name=name
        self.files=[]
        
        
files = []
libs = {}

for src_path, v in list(content.items()):
    lib_name = str(src_path).split("/")[0]
    libs[lib_name] = Lib(lib_name)

for src_path, v in list(content.items()):
    src_path = Path(src_path)
    
    dest = v.get("dest", None)
    assert dest is not None
    weight = v.get("weight", 100)
    title = v.get("title", compute_default_title(src_path.stem))
    toc = v.get("toc", False)
    remove_title = v.get("remove_title", False)
    
    if dest[-1] == "/":
        dest += src_path.name
        
    src_content = read_file(src_path)
    if ".html" in str(src_path):
        src_content = parse_nbdev_output(src_content)
    dest = dest.replace(".html", ".md")
    dest = DEST_FOLDER / dest
    
    out = str(dest)
    out = out[7:]
    out = out.replace(".md", "")
    in_ = str(src_path)
    in_ = in_.replace("pymemri/docs","pymemri")
    
    link_in2out[f"/{in_}"] = f"{out}"
#     print(src_path)
    file = File(src_content, src_path, dest, title, weight)
    libs[file.lib].files.append(file)
    files.append(File(src_content, src_path, dest, title, weight, toc, remove_title))
    continue

#     <a href="/pymemri/pod.client.html#PodClient">
#     <a href="/component-architectures/plugins/pod_client#PodClient">

def replace_links(f, link_in2out):
    # nbdev
    for i, out in link_in2out.items():
        
        i_lib = [x for x in i.split("/") if x != ""][0]
        if f.lib == i_lib:
            if ".html" in str(f.src):
                if f'<a href="{i}' in f.content:
                    f.content = re.sub(f'<a href="{i}', f'<a href="{out}', f.content)

    markdown_links = re.findall('\[[^\]]*\]\(\.[^\)]*\)', f.content)
    patterns = set()

    for markdown_link in markdown_links:
        title = re.search(r"\[[^\]]*\]", markdown_link).group()[1:-1]
        link_relative_address = re.search(r"\([^\)]*\)", markdown_link).group()[1:-1]

        # one for ./ one for .md
        levels_lower = link_relative_address.count(".") - 2
        src_file_location = str(f.src).rsplit("/", 1)[0]
        absolute_location_excluding_link = src_file_location.rsplit("/", levels_lower)[0]
        absolute_link_location = absolute_location_excluding_link + link_relative_address.replace(".md", "").replace(".", "")
        
        if "#" in absolute_link_location:
            absolute_link_location, header_id = absolute_link_location.split("#")
            print(absolute_link_location, header_id)
            pass
        else:
            header_id=""
        
        link_in2out_structure = f"/{absolute_link_location}.md"
        pattern = f'[{title}]({link_relative_address})'

        if link_in2out_structure in link_in2out:
            out = link_in2out[link_in2out_structure]
            patterns.add(pattern)

            if not (pattern in f.content) and pattern not in patterns:
                print(f"Failed to replace {pattern} for {f.src}")
            # replace
            f.content = f.content.replace(pattern,
                               f'[{title}]({out})') 
        else:
            print(f"could not find {link_in2out_structure} {markdown_link} IN IN2OUT, ORIGIN {f.src}")
            f.content = f.content.replace(pattern, f'{title}') 

def replace_image_links(f):
    for k, v in assets_map.items():
        if k in f.content:
            print(f.src, k, v)
        f.content = f.content.replace(k, v)   
  
for f in files:
    replace_links(f, link_in2out)

for f in files:
    replace_mermaid(f)
    replace_image_links(f)
    
print()
    
for f in files:
    write_file(f)
